@@ -12,6 +12,13 @@ around 'get_namespaces' => sub {
     return grep { $_ ne CHI_Meta_Namespace } $self->$orig(@_);
 };
 
+around 'remove' => sub {
+    my ( $orig, $self, $key ) = @_;
+
+    # Call transform_key before passing to remove
+    return $self->$orig( $self->transform_key($key) );
+};
+
 1;
 
 __END__
